import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the Areachart
const xAxisTickFormat = (number) => format('.3s')(number);

const render = (data) => {
  const xValue = (d) => d.Country;
  const xAxisLabel = 'Country';

  const yValue = (d) => d.LandArea;
  const yAxisLabel = 'LandArea';

  const margin = {
    top: 10,
    right: 40,
    bottom: 108,
    left: 150,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  // Determing the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  // Determing the Scale for Y axis
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  // POsitioning the svg element
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );
  // Assigning values to x axis
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Assigning values to y axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -93)
    .attr('x', -innerHeight / 2)
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 75)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);
  // To draw the line on the curve   and adding it to svg
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis);
  // To draw the area under the curve and adding it to svg

  g.append('path')
    .attr('class', 'line-chart')
    .attr('d', lineGenerator(data));
};
// Loading and Parsing CSV data
csv(
  'https://raw.githubusercontent.com/PadmajaKodatii/SDV_Project/main/population_by_country_2020.csv',
).then((data) => {
  data.forEach((d) => {
    d.LandArea = +d.LandArea;
    //Formatting to retrieve year from the date
    d.Country = +d.Country;
  });
  render(data);
});
